https://github.com/jarun/nnn/blob/master/nnn.c#L358
https://github.com/jarun/nnn/blob/master/nnn.c#L1267
https://github.com/jarun/nnn/blob/master/nnn.c#L1617
Most of the functions in nnn are targeted to be high-performance like these.
For example, I don't think your getorder() could possibly be faster than __builtin_ctz(), or even a wrapper around ffs(). You also hardcoded the size of size_t to 32 bits, so it's not more portable than those options.
Edit: I checked, ffs is a tiny branch-free routine in glibc:
(gdb) disassemble ffs
Dump of assembler code for function ffsl:
0x0007e9d0 <+0>: mov $0xffffffff,%edx
0x0007e9d5 <+5>: bsf 0x4(%esp),%eax
0x0007e9da <+10>: cmove %edx,%eax
0x0007e9dd <+13>: add $0x1,%eax
0x0007e9e0 <+16>: ret
End of assembler dump.avoided it to stay out of compiler-specific stuff. we do have plans to replace getorder() with ffsl().
Perhaps I would be more accurate if I say nnn is faster by design. Movement of data around memory is minimal. No redundant bytes are allocated. We use quicksort and optimize further by pushing non-matches down right away so they never appear in a filter comparison again. And of course, using non-lib custom functions enable using static linkage, having a controlled binary size and removing redundant checks/processing because the limits and borderline cases are known.
> avoid div instructions in favour of floating point multiply.
> Is this still faster on recent CPUs?
NB - >>[Taking this question out of context]<< - looks like the answer is "no". From a quick test:- intel Broadwell: 64-bit integer div is fastest, float div is 69% slower, float mul is 50% slower;
- AMD Ryzen: 64-bit integer div is fastest, float div is 47% slower, float mul is 60% slower;
- ARM11 (Raspberry Zero): 64-bit integer div is fastest, float div is 138% slower, float mul is 159% slower.
quick-and-dirty test used: https://pastebin.com/raw/4XqnuXL5
To get the size of a file we multiply only once with this factor.
What is significant is the while() loop where the iterations increase with the file size. There's no mul or div in it giving the API a reasonable performance benefit.